"""
Description
===========

Classes for Shelly(TM) IoT product family.

These classes implement the interface between Shelly devices and
the Juham framework. They listen appropriate messages in the juham 
mqtt network  and control their Shelly devices accordingly, as well as 
listen the sensor data generated by Shellys and publish it in the standard
Juham format.

"""

from .shelly_plugin import ShellyPlugin
from .shellymotion import ShellyMotion
from .shellyds18B20 import ShellyDS18B20
from .shelly1g3 import Shelly1G3
from .shellyplus1 import ShellyPlus1
from .shellypro3em import ShellyPro3EM
from .shelly import Shelly
from .shellypro3 import ShellyPro3
from .shellydht22 import ShellyDHT22

__all__ = [
    "ShellyPlugin",
    "ShellyMotion",
    "ShellyDS18B20",
    "Shelly1G3",
    "ShellyPro3EM",
    "Shelly",
    "ShellyPro3",
    "ShellyPlus1",
    "ShellyDHT22",
]
